﻿using AcmUtilities.Enums;
using AcmUtilities.Helper;
using Microsoft.Extensions.Configuration;
using MigrageAudienceHash;
using MigrageAudienceHash.DB.mongo;
using MigrageAudienceHash.DB.postgres;
using MigrageAudienceHash.Service;
using Npgsql;
using System.Diagnostics;
using System.Globalization;
using System.Net.Mail;

// Build a config object, using env vars and JSON providers.
var config = new ConfigurationBuilder()
    .AddJsonFile("appsettings.json")
    .AddEnvironmentVariables()
    .Build();

AppContext.SetSwitch("Npgsql.EnableLegacyTimestampBehavior", true);

var settings = config.GetRequiredSection("Configuration").Get<ConfigSettings>();


var mongoContext = new MongoContext(new MongoDB.Driver.MongoClient(settings.MongoDbConnectionString), settings.MongoDbName);
var postgresConnection = new NpgsqlConnection(settings.TimeScaleDbConnectionString);

var analyticViewService = new AnalyticViewService(postgresConnection);
var audienceObjectService = new AudienceObjectService(mongoContext);

DateTime fromDate;
if (!DateTime.TryParseExact(settings.FromTimeline, "yyyyMMdd", CultureInfo.InvariantCulture, DateTimeStyles.None, out fromDate))
{
    Console.WriteLine("from timeline not valid!");
    return;
}
var nowDate = DateTime.Now.Date;
int pageSize = 500;
var timeline = 0;
try
{
    while (fromDate.Date < nowDate)
    {
        await ProcessByDay(fromDate, analyticViewService, audienceObjectService, settings.IsMakeNewData == "1");
        fromDate = fromDate.AddDays(1);
    }
}
catch (Exception ex)
{
    Console.WriteLine(timeline + ": " + ex.Message);
}
Console.WriteLine("DONE!");
Console.Read();

async Task ProcessByDay(DateTime date, AnalyticViewService analyticViewService, AudienceObjectService audienceObjectService, bool isMakeNewData = true)
{
    int totalAnalyticRow = 0;
    int pageIndex = 0;
    timeline = CommonHelper.DateToTimeline(date, TimelineType.day);
    Console.WriteLine(timeline + ": start " + DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss"));
    var watch = Stopwatch.StartNew();
    var analyticViews = await analyticViewService.GetData(date, pageIndex, pageSize);
    while (analyticViews.Count > 0)
    {
        totalAnalyticRow += analyticViews.Count;
        var audienceObjects = BuildAudienceObject(analyticViews, timeline);

        //push into Mongo
        if (isMakeNewData)
        {
            await audienceObjectService.InsertManyAsync(audienceObjects);
        }
        else
        {
            foreach (var audienceObject in audienceObjects)
            {
                await audienceObjectService.UpdateOrInsertOneAsync(audienceObject);
            }
        }        

        //next page in this timeline
        pageIndex++;
        analyticViews = await analyticViewService.GetData(date, pageIndex, pageSize);
    }
    watch.Stop();
    Console.WriteLine(timeline + ": finished, total rows: " + totalAnalyticRow + " in " + watch.ElapsedTicks / Stopwatch.Frequency + "s");
}

List<AudienceObject> BuildAudienceObject(List<AnalyticView> analyticViews, int timeline)
{
    var audienceObjects = new List<AudienceObject>();
    //bóc chuỗi macs thành list và group tiếp theo audienceHash
    foreach (var analyticView in analyticViews)
    {
        //group macAddreses into the same audienceHash
        var dicMacByHash = new Dictionary<int, List<string>>();
        var macs = analyticView.audiencies.Split(',');
        foreach (var macAddress in macs)
        {
            var audienceHash = Common.GetHash(macAddress);
            if (dicMacByHash.ContainsKey(audienceHash))
            {
                //append mac into same audienceHash
                var value = dicMacByHash[audienceHash];
                value.Add(macAddress);
                dicMacByHash[audienceHash] = value;
            }
            else
            {
                dicMacByHash.Add(audienceHash, new List<string> { macAddress });
            }
        }
        foreach (var pair in dicMacByHash)
        {
            audienceObjects.Add(new AudienceObject
            {
                timeline = timeline,                
                placeId = analyticView.placeId,
                campaignId = analyticView.campaignId,
                pageCode = analyticView.pageCode,
                audienceHash = pair.Key,
                audiencies = pair.Value
            });
        }
    }
    return audienceObjects;
}
